import argparse

from education_math_homework_generator.util import convert_latex_to_pdf
from education_math_homework_generator.util import remove_temporary_files


def generate_number_lines(number_of_lines=6, start=0, end=20):
    """
    Generates number lines as a tool for practicing mathematics such as addition or subtraction.
    :param number_of_lines: Specify the number of lines to have on the page
    :param start: start value for the number line as an integer
    :param end: end value for the number line as an integer
    :return: contents of the latex document as a string
    """
    lines = [r'\documentclass[letterpaper]{article}',
             r'\usepackage{geometry}',
             r'\geometry{landscape,a4paper,total={170mm,257mm},left=10mm,right=10mm,top=30mm}',
             r'\usepackage{tikz}',
             r'\usepackage{amsmath}',
             r'\usetikzlibrary{arrows}',
             r'\begin{document}',
             r'\begin{LARGE}',
             r'']

    numbers = ','.join([str(x) for x in range(start, end + 1)])
    for i in range(number_of_lines):
        lines.append(r'')
        lines.append(r'{\Large $-$}')
        lines.append(r'\begin{tikzpicture}')
        lines.append(r'\draw[latex-latex, thick] ' + '({},0) -- ({},0) ;'.format(start - 1, end + 1))
        lines.append(r'\foreach \x in  {' + numbers + '}')
        lines.append(r'\draw[shift={(\x,0)},color=black, thick] (0pt,3pt) -- (0pt,-3pt);')
        lines.append(r'\foreach \x in  {' + numbers + '}')
        lines.append(r'\draw[shift={(\x,0)},color=black, thick] (0pt,0pt) -- (0pt,-3pt) node[below] ')
        lines.append(r'{\textbf{\x}};')
        lines.append(r'\end{tikzpicture}')
        lines.append(r'{\Large $+$}')
        lines.append(r'\\')
        lines.append(r'\vspace*{50px}')
        lines.append(r'')

    lines.append(r'\end{LARGE}')
    lines.append(r'\end{document}')

    return '\n'.join(lines)


def parse_arguments():
    """
    Parse user arguments to modify how the document is generated for number lines
    :return: parsed args passed by the user or defaults defined below
    """
    parser = argparse.ArgumentParser(description='Generate a numberline to practice Addition/Subtraction')
    parser.add_argument('--start', default=0, type=int, help='integer to start the number line')
    parser.add_argument('--end', default=20, type=int, help='integer to end the number line')
    parser.add_argument('--numlines', default=5, metavar='N', type=int, help='number of lines to generate')
    parser.add_argument('--filename', default='numberline_01.tex', help='filename to generate')
    return parser.parse_args()


def generate_number_lines_pdf(args):
    """
    Takes the parsed arguments, generates appropriate latex string, converts it a pdf, and cleans up any temporary files
    :param args: parsed arguments that define how to generate the document
    """
    contents = generate_number_lines(number_of_lines=args.numlines, start=args.start, end=args.end)
    convert_latex_to_pdf(args.filename, contents=contents, view=True)
    remove_temporary_files(args.filename)


if __name__ == "__main__":
    generate_number_lines_pdf(parse_arguments())



import os


def remove_temporary_files(filename, extensions=('.tex', '.aux', '.log')):
    for extension in extensions:
        if os.path.isfile(filename.replace('.tex', extension)):
            os.unlink(filename.replace('.tex', extension))


def open_pdf(pdf_filename, pdf_viewer='evince'):
    os.system('{} {} & '.format(pdf_viewer, pdf_filename))


def write_latex_data_to_file(filename, contents):
    with open(filename, 'w') as output_file:
        output_file.write(contents)


def convert_latex_to_pdf(filename, command='pdflatex', options='-interaction nonstopmode', contents=None, view=False):
    if contents is not None:
        write_latex_data_to_file(filename, contents)

    full_command_line = '{} {} {}'.format(command, options, filename)

    pdf_filename = filename.replace('.tex', '.pdf')
    os.system(full_command_line)

    if view:
        open_pdf(pdf_filename)



import argparse

from education_math_homework_generator.util import convert_latex_to_pdf
from education_math_homework_generator.util import remove_temporary_files


def generate_table(start_int=0, end_int=10, table_type='Addition'):
    """
    Generates reference tables for learning early childhood mathematics
    :param start_int: start value for the table as an integer
    :param end_int: end value for the table as an integer
    :param table_type: type of table to generate. options are Addition, Subtraction, or Multiplication
    :return:
    """
    lines = [r'\documentclass{article}',
             r'\usepackage{geometry}',
             r'\geometry{landscape,a4paper,total={170mm,257mm},left=10mm,right=10mm,top=10mm}',
             r'\usepackage{amsmath}',
             r'\usepackage{amsfonts}',
             r'\usepackage{amssymb}',
             r'\usepackage{dcolumn}',
             r'\newcolumntype{2}{D{.}{}{2.0}}',
             r'\begin{document}',
             r'\begin{large}',
             r'\begin{center}',
             r'{\Large ' + table_type + r' Table version 0.1\par}',
             r'\vspace*{25px}',
             r'\renewcommand\arraystretch{1.3}',
             r'\setlength\doublerulesep{0pt}',
             r'\begin{tabular}{r||*{' + str(end_int - start_int + 1) + '}{3|}}']

    operator = {'Addition': r'$+$',
                'Subtraction': r'$-$',
                'Multiplication': r'$\times$'}

    lines.append(operator[table_type] + ''.join([' & {} '.format(x) for x in range(start_int, end_int + 1)]) + r'\\')
    lines.append('\hline\hline')
    for i in range(start_int, end_int + 1):
        if table_type == 'Addition':
            lines.append(str(i) + ''.join([' & {} '.format(x + i) for x in range(start_int, end_int + 1)]) + r'\\')
        if table_type == 'Subtraction':
            lines.append(str(i) + ''.join([' & {} '.format(x - i) for x in range(start_int, end_int + 1)]) + r'\\')
        if table_type == 'Multiplication':
            lines.append(str(i) + ''.join([' & {} '.format(x * i) for x in range(start_int, end_int + 1)]) + r'\\')
        lines.append('\hline')

    lines.append(r'\end{tabular}')
    lines.append(r'\end{center}')
    lines.append(r'\end{large}')
    lines.append(r'\end{document}')

    return '\n'.join(lines)


def parse_arguments():
    """
    Parse user arguments to modify how the document is generated for tables
    :return: parsed args passed by the user or defaults defined below
    """
    operators = ('Addition',
                 'Subtraction',
                 'Multiplication')

    parser = argparse.ArgumentParser(description='Generate a numberline to practice Addition/Subtraction')
    parser.add_argument('--start_int', default=1, type=int, help='starting integer')
    parser.add_argument('--end_int', default=10, type=int, help='ending integer')
    parser.add_argument('--table_type', default='Multiplication', help='table type')
    parser.add_argument('--filename', default='table_01.tex', help='filename to generate')
    args = parser.parse_args()

    max_int = 26
    assert args.table_type in operators, '{} not a valid table_type, only {}'.format(args.table_type, ', '.join([x for x in operators]))
    assert args.end_int > args.start_int, 'start_int must be less than end_int'
    assert args.end_int - args.start_int < max_int, 'number of columns [end_int - start_int] must be less than {} to fit on the page'.format(max_int)
    return args


def generate_problems_pdf(args):
    """
    Takes the parsed arguments, generates appropriate latex, converts it a pdf, and cleans up any temporary files
    :param args: parsed arguments that define how to generate the document
    """
    contents = generate_table(start_int=args.start_int, end_int=args.end_int, table_type=args.table_type)
    convert_latex_to_pdf(args.filename, contents=contents, view=True)
    remove_temporary_files(args.filename)


if __name__ == "__main__":
    generate_problems_pdf(parse_arguments())



#!/usr/bin/env python

"""Tests for 'education_math_homework_generator.gen_table' package."""

from education_math_homework_generator.gen_table import generate_table
from education_math_homework_generator.util import convert_latex_to_pdf
from education_math_homework_generator.util import remove_temporary_files


def test_generate_table():
    contents = generate_table(start_int=0, end_int=10, table_type='Addition')
    assert contents is not None


def test_generate_addition_table_to_pdf():
    filename = 'test_generate_addition_table_to_pdf.tex'
    contents = generate_table(start_int=0, end_int=10, table_type='Addition')
    convert_latex_to_pdf(filename=filename, contents=contents, view=True)
    remove_temporary_files(filename)


def test_generate_subtraction_table_to_pdf():
    filename = 'test_generate_subtraction_table_to_pdf.tex'
    contents = generate_table(start_int=0, end_int=10, table_type='Subtraction')
    convert_latex_to_pdf(filename=filename, contents=contents, view=True)
    remove_temporary_files(filename)


def test_generate_multiplication_table_to_pdf():
    filename = 'test_generate_multiplication_table_to_pdf.tex'
    contents = generate_table(start_int=0, end_int=10, table_type='Multiplication')
    convert_latex_to_pdf(filename=filename, contents=contents, view=True)
    remove_temporary_files(filename)



import argparse
import random

from education_math_homework_generator.util import convert_latex_to_pdf
from education_math_homework_generator.util import remove_temporary_files


def generate_problems(number_of_problems=2, maximum_integer=50, problem_type='Addition'):
    """
    Generates random example math problems in latex format for practicing Addition, Subtraction, or Multiplication
    :param number_of_problems: defines how many problems to generate
    :param maximum_integer: defines the maximum integer possible during the generation of problems
    :param problem_type: type of problems to generate. options are Addition, Subtraction, or Multiplication
    :return: contents of the latex document as a string
    """
    lines = [r'\documentclass{article}',
             r'\usepackage{amsmath}',
             r'\usepackage{amsfonts}',
             r'\usepackage{amssymb}',
             r'\usepackage{multicol}',
             r'\begin{document}', r'{\Large ' + problem_type + r' practice version 0.1\par}',
             r'{\large using max integer = ' + str(maximum_integer) + r'\par}',
             r'\begin{multicols}{2}',
             r'\begin{large}',
             r'\begin{enumerate}']

    operator = {'Addition': '+',
                'Subtraction': '-',
                'Multiplication': '*'}

    for i in range(number_of_problems):
        lines.append(r'\item')
        lines.append(r'\begin{tabular}{lr}')
        int_1 = random.randint(1, maximum_integer)
        int_2 = random.randint(1, maximum_integer)
        if int_1 < int_2:
            int_2, int_1 = int_1, int_2
        int_1_string = ' '.join(x for x in str(int_1))
        int_2_string = ' '.join(x for x in str(int_2))
        lines.append(r' & ' + int_1_string + r'\\')
        lines.append(operator[problem_type] + r'& ' + int_2_string + r'\\')
        lines.append(r'\hline')
        lines.append(r' &    \\')
        lines.append(r'\end{tabular}')
        lines.append(r'\vspace*{50px}')

    lines.append(r'\end{enumerate}')
    lines.append(r'\end{large}')
    lines.append(r'\end{multicols}')
    lines.append(r'\end{document}')

    return '\n'.join(lines)


def parse_arguments():
    """
    Parse user arguments to modify how the document is generated for problem generation
    :return: parsed args passed by the user or defaults defined below
    """
    operators = ('Addition',
                 'Subtraction',
                 'Mixed')

    parser = argparse.ArgumentParser(description='Generate a numberline to practice Addition/Subtraction')
    parser.add_argument('--maximum_integer', default=10, type=int, help='maximum integer to use in generation')
    parser.add_argument('--numproblems', default=10, type=int, help='number of problems to generate')
    parser.add_argument('--problemtype', default='Addition', help='number of lines to generate')
    parser.add_argument('--filename', default='math_homework_01.tex', help='filename to generate')
    args = parser.parse_args()

    assert args.problemtype in operators, '{} not a valid problemtype, only {}'.format(args.problemtype, ', '.join([x for x in operators]))
    return args


def generate_problems_pdf(args):
    """
    Takes the parsed arguments, generates appropriate latex, converts it a pdf, and cleans up any temporary files
    :param args: parsed arguments that define how to generate the document
    """
    contents = generate_problems(number_of_problems=args.numproblems, maximum_integer=args.maximum_integer, problem_type=args.problemtype)
    convert_latex_to_pdf(args.filename, contents=contents, view=True)
    remove_temporary_files(args.filename)


if __name__ == "__main__":
    generate_problems_pdf(parse_arguments())



#!/usr/bin/env python

"""Tests for 'education_math_homework_generator.util' package."""

from education_math_homework_generator.util import convert_latex_to_pdf
from education_math_homework_generator.util import remove_temporary_files
from education_math_homework_generator.util import write_latex_data_to_file


def test_write_latex_data_to_file():
    filename = 'test_write.tex'
    contents = r"""
    \documentclass{article}
    \begin{document}
    Test page
    \end{document}
    """
    write_latex_data_to_file(filename=filename, contents=contents)
    with open(filename) as fileio:
        assert fileio.read() == contents
    remove_temporary_files(filename)


def test_convert_latex_to_pdf():
    filename = 'test_convert.tex'
    contents = r"""
    \documentclass{article}
    \begin{document}
    Test page
    \end{document}
    """
    convert_latex_to_pdf(filename=filename, contents=contents)
    remove_temporary_files(filename)



#!/usr/bin/env python

"""Tests for 'education_math_homework_generator.gen_add_subtract' package."""

from education_math_homework_generator.gen_add_subtract import generate_problems
from education_math_homework_generator.util import convert_latex_to_pdf
from education_math_homework_generator.util import remove_temporary_files


def test_generate_problems_addition():
    contents = generate_problems(number_of_problems=10, maximum_integer=50, problem_type='Addition')
    assert contents.count('+') == 10


def test_generate_problems_addition_large_int():
    contents = generate_problems(number_of_problems=10, maximum_integer=1000, problem_type='Addition')
    assert contents.count('+') == 10


def test_generate_problems_subtraction():
    contents = generate_problems(number_of_problems=10, maximum_integer=50, problem_type='Subtraction')
    assert contents.count('-') == 10


def test_generate_problems_to_pdf():
    filename = 'test_generate_problems_and_convert.tex'
    contents = generate_problems(number_of_problems=10, maximum_integer=10, problem_type='Addition')
    convert_latex_to_pdf(filename=filename, contents=contents, view=True)
    remove_temporary_files(filename)



#!/usr/bin/env python

"""Tests for 'education_math_homework_generator.util' package."""

from education_math_homework_generator.gen_number_line import generate_number_lines
from education_math_homework_generator.util import convert_latex_to_pdf
from education_math_homework_generator.util import remove_temporary_files


def test_gen_number_line():
    contents = generate_number_lines(number_of_lines=5, start=0, end=20)
    assert contents is not None


def test_gen_number_line_to_pdf():
    filename = 'test_numberline.tex'
    contents = generate_number_lines(number_of_lines=5, start=0, end=20)
    convert_latex_to_pdf(filename=filename, contents=contents, view=True)
    remove_temporary_files(filename)
